"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import json
import platform
import sys
from collections import OrderedDict, namedtuple
from typing import Dict, List, Mapping, Optional, Sequence, Tuple, Union, cast

import pkg_resources

from .__version__ import __author__, __copyright__, __email__, __license__, __version__


BasicEnvInfo = namedtuple("BasicEnvInfo", "uname py_implementation py_version")


class Key:
    UNAME = "uname"
    PLATFORM = "platform"
    PYTHON = "Python"
    PYTHON_IMPLEMENTATION = "python_implementation"
    PYTHON_VERSION = "python_version"


def get_uname(verbosity_level: int = 0) -> str:
    uname = platform.uname()

    if verbosity_level == 0:
        return f"{uname.system}"
    if verbosity_level == 1:
        return f"{uname.system} {uname.release}"

    return f"{uname.system} {uname.release} {uname.machine}"


def get_envinfo(
    packages: Optional[Sequence[str]] = None, verbosity_level: int = 0
) -> Dict[str, str]:
    envinfo = {
        Key.UNAME: get_uname(verbosity_level),
        Key.PYTHON_IMPLEMENTATION: platform.python_implementation(),
        Key.PYTHON_VERSION: platform.python_version(),
    }

    system = platform.system()
    if system == "Linux":
        try:
            from distro import linux_distribution

            envinfo[Key.PLATFORM] = " ".join(linux_distribution()[:2])
        except ImportError:
            pass
    elif system == "Windows":
        envinfo[Key.PLATFORM] = " ".join(platform.win32_ver()[:3])
    elif system == "Darwin":
        mac_ver = platform.mac_ver()
        envinfo[Key.PLATFORM] = " ".join([mac_ver[0], mac_ver[2]])

    if not packages:
        return envinfo

    for pkg in packages:
        if not pkg:
            continue

        try:
            envinfo[pkg] = pkg_resources.get_distribution(pkg).version
        except pkg_resources.DistributionNotFound:
            envinfo[pkg] = "not installed"

    return envinfo


def _pop_basic_envinfo(envinfo: Dict[str, str]) -> BasicEnvInfo:
    return BasicEnvInfo(
        uname=envinfo.pop(Key.UNAME),
        py_implementation=envinfo.pop(Key.PYTHON_IMPLEMENTATION),
        py_version=envinfo.pop(Key.PYTHON_VERSION),
    )


def _dumps_markdown(envinfo: Dict[str, str]) -> str:
    import pytablewriter as ptw

    basic_envinfo = _pop_basic_envinfo(envinfo)
    matrix = [
        [Key.UNAME, basic_envinfo.uname],
        [Key.PYTHON, f"{basic_envinfo.py_implementation} {basic_envinfo.py_version}"],
    ] + [[key, value] for key, value in envinfo.items()]
    writer = ptw.MarkdownTableWriter(headers=["Module", "Version"], value_matrix=matrix, margin=1)

    return writer.dumps()


def _dumps_json(envinfo: Dict[str, str]) -> str:
    basic_envinfo = _pop_basic_envinfo(envinfo)
    outputs = {
        Key.UNAME: basic_envinfo.uname,
        Key.PYTHON: f"{basic_envinfo.py_implementation} {basic_envinfo.py_version}",
    }
    outputs.update(envinfo)

    return json.dumps(outputs, indent=4)


def dumps(
    packages: Optional[Sequence[str]] = None,
    format: Optional[str] = None,
    additional_envinfo: Optional[Mapping[str, str]] = None,
    verbosity_level: int = 0,
) -> str:
    envinfo: Dict[str, str] = OrderedDict()

    if additional_envinfo is not None:
        envinfo.update(additional_envinfo)

    envinfo.update(get_envinfo(packages, verbosity_level=verbosity_level))

    format_name = ""
    if format:
        format_name = format.strip().casefold()

    if format_name == "markdown":
        try:
            return _dumps_markdown(envinfo)
        except ImportError:
            print(
                "required modules not installed. try to install dependencies with:\n"
                "    pip install envinfopy[markdown]\n",
                file=sys.stderr,
            )
    elif format_name == "json":
        return _dumps_json(envinfo)

    basic_envinfo = _pop_basic_envinfo(envinfo)
    lines = [
        f"{Key.UNAME}: {basic_envinfo.uname}",
        f"{Key.PYTHON} version: {basic_envinfo.py_implementation} {basic_envinfo.py_version}",
    ]
    lines.extend([f"{key} version: {value}" for key, value in envinfo.items()])

    if format_name == "itemize":
        return "\n".join(f"- {line}" for line in lines)

    return "\n".join(lines)



import re
import sys
from itertools import combinations

import pytest

import envinfopy
from envinfopy import Key


RE_VERSION = re.compile(r"[0-9]\.[0-9]+\.[0-9]+", re.MULTILINE)


class Test_get_uname:
    def test_normal(self):
        value_v0 = envinfopy.get_uname(verbosity_level=0)
        assert len(value_v0) > 0

        value_v1 = envinfopy.get_uname(verbosity_level=1)
        assert len(value_v1) > len(value_v0)

        value_v2 = envinfopy.get_uname(verbosity_level=2)
        assert len(value_v2) > len(value_v1)


class Test_get_envinfo:
    def test_normal(self):
        envinfo = envinfopy.get_envinfo(["pytest", "envinfopy"])
        print(envinfo)

        assert len(envinfo["uname"]) > 0
        assert RE_VERSION.search(envinfo[Key.PYTHON_VERSION])
        assert RE_VERSION.search(envinfo["pytest"])
        assert RE_VERSION.search(envinfo["envinfopy"])

    def test_abnormal_not_installed(self):
        envinfo = envinfopy.get_envinfo(["not-installed-pkg"])
        print(envinfo)

        assert len(envinfo["uname"]) > 0
        assert RE_VERSION.search(envinfo[Key.PYTHON_VERSION])
        assert envinfo["not-installed-pkg"] == "not installed"

    def test_abnormal_empty(self):
        envinfo = envinfopy.get_envinfo([""])

        assert envinfo == envinfopy.get_envinfo()


class Test_dumps:
    @pytest.mark.parametrize(
        ["format"],
        [
            ["text"],
            ["itemize"],
            ["markdown"],
            ["json"],
        ],
    )
    def test_smoke(self, format):
        result_sep = "-" * 40

        output_v0 = envinfopy.dumps(format=format, verbosity_level=0)
        assert len(output_v0) > 20
        assert RE_VERSION.search(output_v0)

        output_v1 = envinfopy.dumps(format=format, verbosity_level=1)
        print(f"{output_v0}\n\n{output_v1}\n{result_sep}", file=sys.stderr)
        assert len(output_v1) > 20
        assert len(output_v1) >= len(output_v0)
        assert RE_VERSION.search(output_v1)

        output_v2 = envinfopy.dumps(format=format, verbosity_level=2)
        print(f"{output_v1}\n\n{output_v2}\n{result_sep}", file=sys.stderr)
        assert len(output_v2) > 20
        assert len(output_v2) >= len(output_v1)
        assert RE_VERSION.search(output_v2)

    @pytest.mark.parametrize(
        ["lhs", "rhs"],
        list(combinations(["text", "itemize", "markdown", "json"], 2)),
    )
    def test_smoke_format(self, lhs, rhs):
        lhs_text = envinfopy.dumps(format=lhs)
        rhs_text = envinfopy.dumps(format=rhs)

        print(f"[{lhs}]\n{lhs_text}\n\n[{rhs}]\n{rhs_text}")

        assert envinfopy.dumps(format=lhs) != envinfopy.dumps(format=rhs)

    def test_smoke_additional_envinfo(self):
        add_key = "hoge"
        add_version = "1.2.3"

        output = envinfopy.dumps(additional_envinfo={add_key: add_version})
        assert re.search(add_key, output, re.MULTILINE)
        assert re.search(re.escape(add_version), output, re.MULTILINE)



#!/usr/bin/env python3

"""
.. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
"""

import argparse
from textwrap import dedent

from . import dumps
from .__version__ import __version__


def parse_option() -> argparse.Namespace:
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog=dedent(
            """\
            Issue tracker: https://github.com/thombashi/envinfopy/issues
            """
        ),
    )
    parser.add_argument("-V", "--version", action="version", version=f"%(prog)s {__version__}")
    parser.add_argument("-v", "--verbose", dest="verbosity_level", action="count", default=1)

    parser.add_argument(
        "packages",
        nargs="+",
        help="package names to extract versions",
    )
    parser.add_argument(
        "--format",
        choices=["text", "markdown", "json", "itemize"],
        default="text",
        help="output format",
    )

    return parser.parse_args()


def main() -> None:
    ns = parse_option()

    print(dumps(ns.packages, ns.format, verbosity_level=ns.verbosity_level))


if __name__ == "__main__":
    main()



import re
import sys

import pytest
from subprocrunner import SubprocessRunner


RE_VERSION = re.compile(r"[0-9]\.[0-9]+\.[0-9]+", re.MULTILINE)


class Test_cli:
    def test_help(self, tmpdir):
        runner = SubprocessRunner([sys.executable, "-m", "envinfopy", "-h"])
        runner.run()
        assert runner.returncode == 0

    @pytest.mark.parametrize(
        ["format"],
        [
            ["text"],
            ["itemize"],
            ["markdown"],
            ["json"],
        ],
    )
    def test_smoke(self, format):
        runner = SubprocessRunner(
            [sys.executable, "-m", "envinfopy", "--format", format, "pathvalidate", "envinfopy"]
        )
        runner.run()

        print(runner.stdout)

        assert runner.returncode == 0
        assert RE_VERSION.search(runner.stdout)
        assert re.search("envinfopy", runner.stdout, re.MULTILINE)
